import java.util.Scanner;

class TreeNode {
	
	public TreeNode left, right, parent;
	public int key;
	public int size;

	// Simple constructor
	public TreeNode(int newKey, TreeNode newParent) {
		key = newKey;
		left = null;
		right = null;
		parent = newParent;
		size = 1;
	}
	
	// Print to System.out in a bracket-notation like "((0 1 (2)) 3 (4)" recursively
	// No newline printed. Note that building and returning a String would be cleaner
	// but much slower if not done carefully
	public void print() {
		if (left != null) {
			System.out.print("(");
			left.print();
			System.out.print(") ");
		}
		System.out.print(key);
		if (right != null) {
			System.out.print(" (");
			right.print();
			System.out.print(")");
		}
	}
	
	/*** Methods to implement for 1 point */

	// Insert element "key" into the tree
	// You may assume that it is not there yet
	public void insert(int val) {
		size += 1;
		if (val < key) {
			if (left != null)
				left.insert(val);
			else
				left = new TreeNode(val, this);
		} else {
			if (right != null)
				right.insert(val);
			else
				right = new TreeNode(val, this);
		}
	}
	
	// Return if "key" is in the tree, recursively
	public boolean contains(int val) {
		if (val == key)
			return true;
		if (val < key) {
			if (left != null)
				return left.contains(val);
			else
				return false;
		} else {
			if (right != null)
				return right.contains(val);
			else
				return false;
		}
	}

	// Delete "val" from the tree
	// You may assume that the tree contains "val"
	public void delete(int val) {
		size -= 1;
		if (val == key) {
			// We are the node to delete
			// Note that this may not happen in the "-1" node
			// Also, returning from the recursion will fix the subtree sizes
			
			// Found, now it depends on the node type (leaf, one child, two children)
			if (left == null && right == null) {
				// Leaf -> just forget self
				if (parent.right == this)
					parent.right = null;
				else
					parent.left = null;
				return;
			}
			
			if (left == null || right == null) {
				// We only have right child -> forget self, relink the child to parent
				TreeNode child = left;
				if (child == null)
					child = right;
				if (parent.right == this)
					parent.right = child;
				else
					parent.left = child;
				// Also fix the parent reference
				child.parent = parent;
				return;
			}
			
			// Now we have both children, so we need to find the nearest larger value
			// by going one to right and then all the way left
			TreeNode nextLarger = right;
			while (nextLarger.left != null) {
				nextLarger = nextLarger.left;
			}
			
			int nextLargerKey = nextLarger.key;
			// First delete nextLarger instead of us (recursively)
			// Note that we need to start the deletion *here* so that the recursion fixes the tree sizes for us
			this.delete(nextLargerKey);
			// Now "steal" the previous value of nextLarger
			key = nextLargerKey;
			return;
		
		} else {
			// Look in left or right subtrees (we know they exist, as "val" is there)
			if (val < key)
				left.delete(val);
			else 
				right.delete(val);
			// And fix the subtree sizes after you return from recursion 
			size = 1;
			if (left != null)
				size += left.size;
			if (right != null)
				size += right.size;
		}
	}

	/*** Methods to implement for another 1 point */
	
	// Find rank-th smallest element in the tree
	public int findByRank(int rank) {
		// If there is left subtree, recurse if the rank is inside, or substract it from rank
		if (left != null) {
			if (rank < left.size)
				return left.findByRank(rank);
			rank -= left.size;
		}
		if (rank == 0)
			return this.key;
		// Recurse into right subtree, 1 substracted for this node
		if (right == null) {
			assert(false);
		}
		return right.findByRank(rank - 1);
	}
}


/* NB: For the judge to run the program, do not modify the declaration of the class Main or
 *     method main() below. The class has to be declared as "class Main { ... }"
 *     and the method as "public static void main(String[] args) { ... }" */
class Main {

	public static void main(String[] args) {

		// Read from the standard input with java.util.Scanner
		Scanner scanner = new Scanner(System.in);
		boolean run = true;
		
		// We use a "special" smallest value node as a root.
		// Always having at least one node of the tree simplifies a lot of special cases. 
		TreeNode root = new TreeNode(-1, null);

		// A simple loop reading one command at a time
		while (run) {
			String cmd = scanner.next();
			
			if (cmd.equals("I")) {
				int val = scanner.nextInt();
				root.insert(val);
			}
			
			if (cmd.equals("D")) {
				int val = scanner.nextInt();
				root.delete(val);
			}
			
			if (cmd.equals("C")) {
				int val = scanner.nextInt();
				if (root.contains(val)) {
					System.out.println("YES");
				} else {
					System.out.println("NO");
				}
			}
			
			if (cmd.equals("P")) {
				// We are not printing the special "-1" node
				if (root.right == null) {
					System.out.println("EMPTY");
				} else {
					root.right.print();
					System.out.println();
				}
			}
			
			if (cmd.equals("R")) {
				int rank = scanner.nextInt();
				// We look for element number "rank+1" to skip the special "-1" value
				System.out.println(root.findByRank(rank + 1));
			}

			if (cmd.equals("X")) {
				run = false;
			}
		}
		scanner.close();
	}
}
